import React, { useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useAsyncState } from "@/hooks";
import { useSpacing } from "@/styles";
import api from "@/management/api";
import { Box, Button, Container, IconButton, Typography } from "@material-ui/core";
import MaterialTable, { Column } from "material-table";
import { actionsColumn } from "@/management/common/helpers";
import { FileFind, Refresh, StickerCheckOutline, StickerRemoveOutline } from "mdi-material-ui";
import { Page } from "@/pages/base";
import { Actions } from "@/components";
import { BulkActions } from "@/management/common/BulkActions";
import { Async } from "@/components/async";
import { MaterialTableTitle } from "@/management/common/MaterialTableTitle";
import { EditionManagement, EditionManagementProps } from "@/management/edition/manage";
import { Link as RouterLink } from "react-router-dom";
import { route } from "@/routing";
import { AcceptanceActions, AcceptSubmissionDialog, DiscardSubmissionDialog } from "@/components/acceptance-action";
import { ProposalPreview } from "@/components/proposalPreview";
import { InternshipSubmission } from "@/management/api/internship";
import { canAccept, canDiscard, StateLabel } from "@/management/edition/internship/common";
import { createPortal } from "react-dom";

const title = "edition.internships.title";

export const InternshipManagement = ({ edition }: EditionManagementProps) => {
    const { t } = useTranslation("management");
    const [result, setInternshipsPromise] = useAsyncState<InternshipSubmission[]>();
    const [selected, setSelected] = useState<InternshipSubmission[]>([]);
    const spacing = useSpacing(2);

    const updateInternshipList = () => {
        setInternshipsPromise(api.internship.all(edition));
    }

    useEffect(updateInternshipList, []);

    const handleSubmissionAccept = api.internship.approve
    const handleSubmissionDiscard = api.internship.decline

    const AcceptAction = ({ internship }: { internship: InternshipSubmission }) => {
        const [open, setOpen] = useState(false);

        return <>
            <IconButton onClick={ () => setOpen(true) }><StickerCheckOutline /></IconButton>
            { createPortal(
                <AcceptSubmissionDialog onAccept={ comment => handleSubmissionAccept(internship, comment) } label="internship" open={ open } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element,
            ) }
        </>;
    }

    const DiscardAction = ({ internship }: { internship: InternshipSubmission }) => {
        const [open, setOpen] = useState(false);

        return <>
            <IconButton onClick={ () => setOpen(true) }><StickerRemoveOutline /></IconButton>
            { createPortal(
                <DiscardSubmissionDialog onDiscard={ comment => handleSubmissionDiscard(internship, comment) } label="internship" open={ open } onClose={ () => setOpen(false) }/>,
                document.getElementById("modals") as Element,
            ) }
        </>;
    }

    const columns: Column<InternshipSubmission>[] = [
        {
            title: t("internship.column.student"),
            render: internship => <>{internship.intern.name} {internship.intern.surname}</>,
        },
        {
            title: t("internship.column.album"),
            field: "intern.albumNumber",
        },
        {
            title: t("internship.column.type"),
            field: "type.label.pl",
        },
        {
            title: t("internship.column.changed"),
            render: summary => summary.changed.format("yyyy-MM-DD HH:mm")
        },
        {
            title: t("internship.column.status"),
            render: summary => <StateLabel state={ summary.state } />
        },
        actionsColumn(internship => <>
            { canAccept(internship) && <AcceptAction internship={ internship } /> }
            { canDiscard(internship) && <DiscardAction internship={ internship } /> }
            <IconButton component={ RouterLink } to={ route("management:edition_internship", { edition: edition.id || "", internship: internship.id || "" }) }><FileFind /></IconButton>
        </>)
    ];

    return <Page>
        <Page.Header maxWidth="lg">
            <EditionManagement.Breadcrumbs>
                <Typography color="textPrimary">{ t(title) }</Typography>
            </EditionManagement.Breadcrumbs>
            <Page.Title>{ t(title) }</Page.Title>
        </Page.Header>
        <Container maxWidth="lg" className={ spacing.vertical }>
            <Actions>
                <Button onClick={ updateInternshipList } startIcon={ <Refresh /> }>{ t("refresh") }</Button>
            </Actions>
            { selected.length > 0 && <BulkActions>
                <AcceptanceActions label="internship" onAccept={ comment => handleSubmissionAccept(selected, comment) } onDiscard={ comment => handleSubmissionDiscard(selected, comment) } />
            </BulkActions> }
            <Async async={ result } keepValue>{
                internships => <MaterialTable
                    title={ <MaterialTableTitle result={ result } label={ t(title) }/> }
                    columns={ columns }
                    data={ internships }
                    onSelectionChange={ internships => setSelected(internships) }
                    options={ { selection: true, pageSize: 10 } }
                    detailPanel={ internship => <Box m={ 3 }><ProposalPreview proposal={ internship } /></Box> }
                />
            }</Async>
        </Container>
    </Page>
}
